#//-----------------------------------------------
#//TUSB-PIO用ドライバアクセス用補助関数
#//Python 用モジュール
#//
#//2024_03_27　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

# 本モジュールはIC 8255のレジスタ操作を簡易にするための
# 補助関数を収録しています。本モジュール内の関数を呼び出す事で
# 8255のレジスタを直接操作する事なく基本的な入出力機能(モード0)
# を使用できます
# 本モジュールからTUSBPIO.pyを呼び出して操作しますので
# 8255のレジスタを直接設定する必要がある場合にはTUSBPIO
# モジュールを使用してください

#DevIdは装置の設定IDで整数値 0~15

import TUSBPIO
import threading

# 処理モジュール呼び出し用辞書
sDevice = {0:'Device1',1:'Device2'}
sPort = {0:'PortA',1:'PortB',2:'PortC'}
# 逆引き辞書生成
nDevice = {val:key for key, val in sDevice.items()}
nPort = {val:key for key, val in sPort.items()}

lock = threading.Lock()

# デバイスオープン
def Device_Open(DevId):
    ret = TUSBPIO.Tusbpio_Device_Open(DevId)
    if( ret != 0 ):
        raise Exception(TUSBPIO.GetErrMessage1(ret))
    return

# デバイスクローズ
def Device_Close(DevId):
    TUSBPIO.Tusbadsms_Device_Close(DevId)
    return

#I/O入出力方向設定
# DevSel : デバイス1なら'Device1'、デバイス2ならそれ以外　例'Device2'を渡す
# Dir : ディレクションタプル (PortA,PortB,PortC_LOw,PortC_High)
#       各ポート位置に入力なら'In'、出力ならそれ以外文字列 例'Out'を渡す
# PortCはHighビット(7~4bit) Lowビット(3~0bit)に分かれている
def SetDirection(DevId,DevSel,Dir):
    Addr = 3 #コマンドワード
    Data = 0b10000000 #データビット モード選択
    Data += 0b00010000 if(Dir[0] == 'In') else 0b00000000
    Data += 0b00000010 if(Dir[1] == 'In') else 0b00000000
    Data += 0b00000100 if(Dir[2] == 'In') else 0b00000000
    Data += 0b00001000 if(Dir[3] == 'In') else 0b00000000
    with lock:
        if( DevSel == 'Device1' ):
            ret = TUSBPIO.Tusbpio_Dev1_Write(DevId,Addr,Data)
        else:
            ret = TUSBPIO.Tusbpio_Dev2_Write(DevId,Addr,Data)
    if( ret != 0 ):
        raise Exception(TUSBPIO.GetErrMessage2(ret))

#出力データ設定
# DevSel : デバイス1なら'Device1'、デバイス2ならそれ以外　例'Device2'を渡す
# PortSel: ポートを選択 'PortA' 'PortB' 'PortC'(A,B以外ならCとする)
# Data : 書き込むデータ 0~255 (0h00 ~ 0hff)
def SetOutData(DevId,DevSel,PortSel,Data):
    # if PortSel == 'PortA':
    #     Addr = 0
    # elif PortSel == 'PortB':
    #     Addr = 1
    # else:
    #     Addr = 2
    Addr = nPort[PortSel]
    with lock:
        if DevSel == 'Device1':
            ret = TUSBPIO.Tusbpio_Dev1_Write(DevId,Addr,Data)
        else:
            ret = TUSBPIO.Tusbpio_Dev2_Write(DevId,Addr,Data)
    if( ret != 0 ):
        raise Exception(TUSBPIO.GetErrMessage2(ret))
    

#入力読み出し
# DevSel : デバイス1なら'Device1'、デバイス2ならそれ以外　例'Device2'を渡す
# PortSel: ポートを選択 'PortA' 'PortB' 'PortC'(A,B以外ならCとする)
# 戻り値データ　読み込み値 0~255
def ReadInData(DevId,DevSel,PortSel):
    # if PortSel == 'PortA':
    #     Addr = 0
    # elif PortSel == 'PortB':
    #     Addr = 1
    # else:
    #     Addr = 2
    Addr = nPort[PortSel]
    with lock:
        if DevSel == 'Device1':
            ret,Data = TUSBPIO.Tusbpio_Dev1_Read(DevId,Addr)
        else:
            ret,Data = TUSBPIO.Tusbpio_Dev2_Read(DevId,Addr)
    if( ret != 0 ):
        raise Exception(TUSBPIO.GetErrMessage2(ret))
    return Data

