#//-----------------------------------------------
#//TUSB-PIO用ドライバアクセス用API
#//Python 用モジュール
#//
#//2024_03_26　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

from ctypes import *

#デバイス用DLLのロード  
tusbpiodll = cdll.LoadLibrary("TUSBPIO.DLL")

#デバイスドライバ関数の型定義  (これは直接呼ばない)
tusbpiodll.Tusbpio_Device_Open.restype = c_short
tusbpiodll.Tusbpio_Device_Open.argtypes = (c_short,)
tusbpiodll.Tusbpio_Device_Close.restype = None
tusbpiodll.Tusbpio_Device_Close.argtypes = (c_short,)
tusbpiodll.Tusbpio_Dev1_Write.restype = c_short
tusbpiodll.Tusbpio_Dev1_Write.argtypes = (c_short, c_ubyte, c_ubyte )
tusbpiodll.Tusbpio_Dev2_Write.restype = c_short
tusbpiodll.Tusbpio_Dev2_Write.argtypes = (c_short, c_ubyte, c_ubyte )
tusbpiodll.Tusbpio_Dev1_Read.restype = c_short
tusbpiodll.Tusbpio_Dev1_Read.argtypes = (c_short, c_ubyte, POINTER(c_ubyte))
tusbpiodll.Tusbpio_Dev2_Read.restype = c_short
tusbpiodll.Tusbpio_Dev2_Read.argtypes = (c_short, c_ubyte, POINTER(c_ubyte))

#######  補助関数 (これも呼ばない)
def isint(n):
    return type(n) is int

def IsInRange(Val,Min,Max):
    if(isint(Val) == False ):
        return False
    elif(Val < Min):
        return False
    elif(Val > Max):
        return False
    else:
        return True

def IsValidId(DevId):
    return IsInRange(DevId,0,15)

######　機能関数の定義 
######  以下の関数を呼び出してデバイスを操作する
#DevIdは装置の設定ID

# デバイスオープン
def Tusbpio_Device_Open(DevId):
    if( IsValidId(DevId) ):
        return tusbpiodll.Tusbpio_Device_Open(DevId)
    else:
        return 8    #Invalid parameter
    
#デバイスクローズ
def Tusbadsms_Device_Close(DevId):
    if( IsValidId(DevId) ):
        tusbpiodll.Tusbpio_Device_Close(DevId)
    else:
        return 8    #Invalid parameter
    
#8255 IC1レジスタ書き込み
def Tusbpio_Dev1_Write(DevId,Addr,WriteData):
    if( IsValidId(DevId) and IsInRange(Addr,0,3)):
        if(IsInRange(WriteData,0,255)):
            return tusbpiodll.Tusbpio_Dev1_Write(DevId,Addr,WriteData)
    return 8    #Invalid parameter

#8255 IC2レジスタ書き込み
def Tusbpio_Dev2_Write(DevId,Addr,WriteData):
    if( IsValidId(DevId) and IsInRange(Addr,0,3)):
        if(IsInRange(WriteData,0,255)):
            return tusbpiodll.Tusbpio_Dev2_Write(DevId,Addr,WriteData)
    return 8    #Invalid parameter

#8255 IC1レジスタ読み込み
# 戻り値タプル (結果コード、読み込みデータ)
def Tusbpio_Dev1_Read(DevId,Addr):
    tmp = c_ubyte()
    if( IsValidId(DevId) and IsInRange(Addr,0,3)):
        ret = tusbpiodll.Tusbpio_Dev1_Read(DevId,Addr,byref(tmp))
        if( ret != 0 ):
            return ret,0
        else:
            return 0,tmp.value
    else:
        return 8,0    #Invalid parameter

#8255 IC2レジスタ読み込み
# 戻り値タプル (結果コード、読み込みデータ)
def Tusbpio_Dev2_Read(DevId,Addr):
    tmp = c_ubyte()
    if( IsValidId(DevId) and IsInRange(Addr,0,3)):
        ret = tusbpiodll.Tusbpio_Dev2_Read(DevId,Addr,byref(tmp))
        if( ret != 0 ):
            return ret,0
        else:
            return 0,tmp.value
    else:
        return 8,0    #Invalid parameter
    
#戻り値用エラーメッセージ(オープン関数)
ErrStrDic1 = {
            0:"正常終了しました",
            1:"ID番号が不正です",
            3:"すでにデバイスはオープンされています",
            4:"接続されている台数が多すぎます",
            5:"オープンできませんでした",
            6:"デバイスがみつかりません"
}
def GetErrMessage1(retcode):
    if( retcode in ErrStrDic1):
        return ErrStrDic1[retcode]
    else:
        return "不明なエラーです"
    
#戻り値用エラーメッセージ(その他の関数)
ErrStrDic2 = {
            0:"正常終了しました",
            1:"オープンされていません",
            2:"失敗しました"
}
def GetErrMessage2(retcode):
    if( retcode in ErrStrDic2):
        return ErrStrDic2[retcode]
    else:
        return "不明なエラーです"

