#//-----------------------------------------------
#//TUSB-PIO用Pythonサンプルプログラム
#//
#//2024_03_27　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

import PIOWrap                  # TUSB-PIO 処理モジュール

import tkinter as tk
import tkinter.ttk as ttk
from tkinter import messagebox
import threading
import time
from functools import partial

# 処理モジュール呼び出し用辞書
from PIOWrap import sDevice
from PIOWrap import sPort

TusbpioId = 0                   # TUSB-PIOの設定ID

#/// Sub Functions //////////////////////////////

# Widgetの表示を設定する
# stat -> 'Close' , 'OpenStop' , 'OpenRun'
def WidgetState(stat):
    EnClose = tk.NORMAL if( stat == 'Close' ) else tk.DISABLED
    EnOpen = tk.NORMAL if( stat == 'Open' ) else tk.DISABLED
    OpenBtn['state'] = EnClose
    id_combo['state'] = EnClose
    CloseBtn['state'] = EnOpen
    for dev in range(2):
        for port in range(4):
            PortInRBtn[dev][port]['state'] = EnOpen
            PortOutRBtn[dev][port]['state'] = EnOpen
        for port in range(3):
            for bit in range(8):
                PortOutBitDev[dev][port][bit]['state'] = EnOpen
    if stat != 'Close' :
        form.protocol('WM_DELETE_WINDOW',lambda: 'pass' )
    else :
        form.protocol('WM_DELETE_WINDOW',lambda: form.destroy())

# 文字列strを表示する
def ShowInfo(str):
    messagebox.showinfo("",str)

# 出力ビット設定を読み込む
def GetOutBit(DevSel,PortSel):
    ret = 0
    for bit in range(8):
        if OutBitChk[DevSel][PortSel][bit].get():
            ret += ( 1 << bit )
    return ret

# 入力ビットを読みこむ
PortInData = [[0 for _ in range(3)] for _ in range(2)]
def ChkInput():
    global PortInData
    for Dev in range(2):
        for Port in range(3):
            PortInData[Dev][Port] = PIOWrap.ReadInData(TusbpioId,sDevice[Dev],sPort[Port])
    return
# // thread ////////////////////////////
threadrun = True
# 読み込まれた入力ビットパターンを表示に反映させる
def InputSetLoop():
    while threadrun:
        for Dev in range(2):
            for Port in range(3):
                for Bit in range(8):
                    if ( PortInData[Dev][Port] & (1<<Bit) ) == (1<<Bit):
                        PortInBitDev[Dev][Port][Bit]['bg'] = "#FFF000000" #Red
                    else:
                        PortInBitDev[Dev][Port][Bit]['bg'] = "#800000000" #Dark Red
        time.sleep(0.1)
inv_thread = None   #表示更新スレッド

#/// interval timer ///////////////////////////
def TimerLoop():
    while threadrun:
        ChkInput()
        time.sleep(0.1)
tim_thread = None

#/// Button event handlers /////////////////////////
#入出力方向設定ボタンのハンドラ
def DirSet_on(DevSel):
    Dir = (
        PortInOutSel[DevSel][0].get(),
        PortInOutSel[DevSel][1].get(),
        PortInOutSel[DevSel][2].get(),
        PortInOutSel[DevSel][3].get()
    )
    Dev = 'Device1' if DevSel == 0 else 'Device2'
    PIOWrap.SetDirection(TusbpioId,Dev,Dir)
    for port in range(3):
        OutBit_on(DevSel,port)
    return

#出力設定チェックボックスのハンドラ
def OutBit_on(DevSel,PortSel):
    Data = GetOutBit(DevSel,PortSel)
    PIOWrap.SetOutData(TusbpioId,sDevice[DevSel],sPort[PortSel],Data)
    return

#TUSB-PIOデバイスオープン処理
def DevOpen_on():
    global TusbpioId
    global threadrun
    global tim_thread
    global inv_thread
    TusbpioId = id_combo.current()
    try:
        PIOWrap.Device_Open(TusbpioId)
        threadrun = True
        tim_thread = threading.Thread(target=TimerLoop)
        tim_thread.start()
        inv_thread = threading.Thread(target=InputSetLoop)
        inv_thread.start()
        WidgetState('Open')
    except any as ex:
        ShowInfo(ex)

#TUSB-PIOデバイスクローズ処理
def DevClose_on():
    global threadrun
    threadrun = False
    tim_thread.join(timeout=1)
    PIOWrap.Device_Close(TusbpioId)
    WidgetState('Close')

#/// form generator ///////////////////////////
#フォーム生成
form = tk.Tk()
form.geometry('768x372')
form.title('TUSB-PIO Sample Program')
#ウィジットリスト
DevFrame = [[None] for _ in range(2)]
PortFrame = [[None for _ in range(4)] for _ in range(2)]
PortInRBtn = [[None for _ in range(4)] for _ in range(2)]
PortOutRBtn = [[None for _ in range(4)] for _ in range(2)]
PortOutBitDev = [[[None for _ in range(8)] for _ in range(3)] for _ in range(2) ] 
PortInBitDev = [[[None for _ in range(8)] for _ in range(3)] for _ in range(2) ]    # 入力ビット表示 (Canvas)
#ウィジットの値を取得するためのリスト
PortInOutSel = [[tk.StringVar(value = 'In') for _ in range(4) ] for _ in range(2)]      # ディレクション選択 'In' or 'Out'
OutBitChk = [[[tk.BooleanVar() for _ in range(8)] for _ in range(3) ] for _ in range(2)]# 出力ビット設定チェック (Checkbutton)
# Device1 Device2 フレーム
DevFrame[0] = tk.LabelFrame(form, text = 'Device 1')
DevFrame[0].place(relwidth = 0.482,relheight=0.793,relx=0.013,rely=0.027,anchor=tk.NW)
DevFrame[1] = tk.LabelFrame(form, text = 'Device 2')
DevFrame[1].place(relwidth = 0.482,relheight=0.793,relx=0.508,rely=0.027,anchor=tk.NW)

# フレーム内要素生成
for Dev in range(2):
    #button
    DirSetBtn1 = tk.Button(DevFrame[Dev], text = 'Direction Set', command=partial(DirSet_on, DevSel = Dev))
    DirSetBtn1.place(relwidth =0.424,relheight=0.088,relx=0.736,rely=0.05,anchor=tk.CENTER)

    #Port Frames
    PortFrame[Dev][0] = tk.LabelFrame(DevFrame[Dev],text = 'PortA')
    PortFrame[Dev][0].place(relwidth = 0.216, relheight = 0.97, relx = 0.027, rely = 0.0,anchor=tk.NW)
    PortFrame[Dev][1] = tk.LabelFrame(DevFrame[Dev],text = 'PortB')
    PortFrame[Dev][1].place(relwidth = 0.216, relheight = 0.97, relx = 0.27, rely = 0.0,anchor=tk.NW)
    PortFrame[Dev][2] = tk.LabelFrame(DevFrame[Dev],text = 'PortC_Low')
    PortFrame[Dev][2].place(relwidth = 0.216, relheight = 0.7, relx = 0.514, rely = 0.12,anchor=tk.NW)
    PortFrame[Dev][3] = tk.LabelFrame(DevFrame[Dev],text = 'PortC_High')
    PortFrame[Dev][3].place(relwidth = 0.216, relheight = 0.7, relx = 0.757, rely = 0.12,anchor=tk.NW)

    # A Port / B Port
    for AB in (0,1):
        #In Out RadioButtons
        PortInRBtn[Dev][AB] = tk.Radiobutton(PortFrame[Dev][AB], text = 'Input',value = 'In',variable=PortInOutSel[Dev][AB])
        PortInRBtn[Dev][AB].place(relx=0.08,rely=0.05,anchor=tk.W)
        PortOutRBtn[Dev][AB] = tk.Radiobutton(PortFrame[Dev][AB], text = 'Output',value = 'Out',variable=PortInOutSel[Dev][AB])
        PortOutRBtn[Dev][AB].place(relx=0.08,rely=0.15,anchor=tk.W)
        #Checkboxes and Lamps
        for bit in range(8):
            PortOutBitDev[Dev][AB][bit] = tk.Checkbutton(PortFrame[Dev][AB], text = str(bit),variable=OutBitChk[Dev][AB][bit],command = partial(OutBit_on, DevSel= Dev,PortSel=AB))
            PortOutBitDev[Dev][AB][bit].place(relx = 0.08,rely=0.27+0.09*bit,anchor=tk.W)
            PortInBitDev[Dev][AB][bit] = tk.Canvas(PortFrame[Dev][AB],width = 15, height = 15, bg = "#800000000")
            PortInBitDev[Dev][AB][bit].place(width = 15, height = 15, relx = 0.65, rely = 0.27 +0.09*bit,anchor = tk.W)

    # C Port
    for LH in (2,3):
        #In Out RadioButtons
        PortInRBtn[Dev][LH] = tk.Radiobutton(PortFrame[Dev][LH], text = 'Input', value = 'In', variable = PortInOutSel[Dev][LH])
        PortInRBtn[Dev][LH].place(relx=0.08,rely=0.07,anchor=tk.W)
        PortOutRBtn[Dev][LH] = tk.Radiobutton(PortFrame[Dev][LH], text = 'Output', value = 'Out', variable = PortInOutSel[Dev][LH])
        PortOutRBtn[Dev][LH].place(relx=0.08,rely=0.21,anchor=tk.W)
        #Checkboxes and Lamps
        for bit in range(4):
            tport = 2
            tbit = ( LH - 2 ) * 4 + bit
            PortOutBitDev[Dev][tport][tbit] = tk.Checkbutton(PortFrame[Dev][LH], text = str(tbit),variable=OutBitChk[Dev][tport][tbit],command = partial(OutBit_on, DevSel=Dev,PortSel=tport))
            PortOutBitDev[Dev][tport][tbit].place(relx = 0.08,rely=0.38+0.125*bit,anchor=tk.W)
            PortInBitDev[Dev][tport][tbit] = tk.Canvas(PortFrame[Dev][LH],width = 15, height = 15, bg = "#800000000")
            PortInBitDev[Dev][tport][tbit].place(width = 15, height = 15, relx = 0.65, rely = 0.38 +0.125*bit,anchor = tk.W)

# list boxes
id_option = list(range(16))
id_combo = ttk.Combobox (form, values = id_option,state="readonly")
id_combo.place(relwidth= 0.1,relheight =0.06,relx = 0.05,rely=0.9,anchor=tk.W)
id_combo.current(0)

#Open Button
OpenBtn = tk.Button(form, text = 'Dev_Open', command=DevOpen_on)
OpenBtn.place(relwidth =0.1,relheight=0.06,relx=0.2,rely=0.9,anchor=tk.W)
#Close Button
CloseBtn = tk.Button(form, text = 'Dev_Close', command=DevClose_on)
CloseBtn.place(relwidth =0.1,relheight=0.06,relx=0.35,rely=0.9,anchor=tk.W)

#/////////////////////////////////////////////////
        
#//// Start

WidgetState('Close')
form.mainloop()     # フォーム画面表示

